<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		Payplans
* @subpackage	Planaddons
* @contact		support+payplans@eadybytes.in
*/

if(defined('_JEXEC')===false) die();

define('PAYPLANS_PLANADDONS_MODIFIER', "plan_addons");

/**
 * Addon conditions
 */
define('PAYPLANS_PLANADDONS_ONETIME', 501);
define('PAYPLANS_PLANADDONS_EACHRECURRING', 502);

class plgPayplansPlanaddons extends XiPlugin
{
	public $_name = 'Planaddons';

	/**
	 * load plugin's view,controller,model,table and lib
	 */
	public function onPayplansSystemStart()
	{
		//autoload view,controller,model and table
		$dir = dirname(__FILE__).DS.'planaddons';
		PayplansHelperLoader::addAutoLoadFolder($dir.DS.'views', 'view', 'Payplansadmin');
        PayplansHelperLoader::addAutoLoadFolder($dir.DS.'tables', 'table');
        PayplansHelperLoader::addAutoLoadFolder($dir.DS.'models', 'model');
        PayplansHelperLoader::addAutoLoadFolder($dir.DS.'controllers', 'controller', 'Payplansadmin');
        PayplansHelperLoader::addAutoLoadFolder($dir.DS.'lib', '');

		return true;
	}

	/**
	 * set plugin template for front end
	 */
	public function onPayplansViewBeforeRender(XiView $view, $task)
	{
		//add admin-submenu when plugin in enabled
		if (XiFactory::getApplication()->isAdmin() && XiHelperJoomla::getPluginStatus('planaddons', 'payplans')) {
			$controller = XiFactory::getInstance('Planaddons', 'adminController');
			$available = $controller->availableOrNot(false);		
			XiAbstractView::addSubmenus('planaddons');

			if ($available) {
				XiAbstractView::addSubmenus('planaddonsstats');
			}
		}
					
		//render widget on invoice confirm page
		if ($view instanceof PayplanssiteViewInvoice && $task == 'confirm') {
			$itemid	= $view->getModel()->getId();
			if (empty($itemid)) {
				return true;
			}
			
			// Chcek for which plan planaddons will be work
			$invoice  			= PayplansInvoice::getInstance($itemid);
			$invoicePlans 		= $invoice->getPlans();
			
			$model 				= PayplansFactory::getInstance('planaddons', 'model');
			$availableAddons 	= $model->loadAvailableServices($invoicePlans );
			
			if (empty($availableAddons)) {
				return true;
			}
			
			$document = XiFactory::getDocument();
			
			//add upgrade javascript file
			$scriptUrl = PayplansHelperTemplate::mediaURI(dirname(__FILE__).DS.'planaddons'.DS.'tmpl'.DS.'planaddons.js', false);
			$document->addScript($scriptUrl);

			//Get dynamic java script
			$jsScript	=	$this->_getModalJavascript($invoice);
			if($jsScript){
				$document->addScriptDeclaration($jsScript);
			}

			$args 		= array( 'template_content' => $this->params->get('template_content', 'Add Services'));
			$isInModal 	= $this->params->get('addons_in_popup', true);
			
			if ($isInModal) {
				$content = $this->_loadTemplate('invoice_confirm', $args, 'payplans', 'default');
			} else {
				$content = $this->_loadTemplate('invoice_confirm_accordion', $args, 'payplans', 'default');
			}
			
			$contentPosition = $this->params->get('template_position', 'payplans_order_confirm_payment');
			return array($contentPosition => $content);
		}
	
		return true;
	}
	
	public function _getModalJavascript(PayplansInvoice $invoice)
	{
		//add language string to be used in javascript
    	JText::script('COM_PAYPLANS_PLANADDONS_MODAL_TITLE');
    	JText::script('COM_PAYPLANS_PLANADDONS_MODAL_FETCHING_SERVICE_WAIT');
    	
        ob_start(); ?>
		
		var addonservice_user_id =  <?php echo $invoice->getBuyer(); ?>;
		var addonservice_invoice_id =  <?php echo $invoice->getId(); ?>;

        <?php
        $js = ob_get_contents();
        ob_end_clean();
        return $js;
	}

 	/**
 	 * What are the available addons, and already purchased by user
 	 * @param  int $userId    user who is purchasing
 	 * @param  int $invoiceId on which invoice
 	 * @return ajaxresponse available addons
 	 * @since 3.2
 	 */
	public function onPayplansPlanaddonsRequest($userId, $invoiceId)
	{

		$appendTo 		= JFactory::getApplication()->input->get('appendto', 'xiWindowBody');
		$invoice 		= PayplansInvoice::getInstance($invoiceId);
		$invoicePlans 	= $invoice->getPlans();

		//get all services that are applicable on that plans.
		$addonsModal 	= PayplansFactory::getInstance('planaddons', 'model');

		$availableAddons = $addonsModal->loadAvailableServices($invoicePlans);

		//get all purhcased service on this invoice, user may want to modify his purchase.
		$servicesStatsModal = PayplansFactory::getInstance('planaddonsstats', 'model');

		//array of id's that are already purchased
		$purchasedServices = $servicesStatsModal->getPurchasedServices($invoice->getId());

		$forcefulDefault 	= $this->params->get('forceful_default', 0);

		$args = array('availableServices' => $availableAddons , 'invoice' => $invoice, 'purchasedServices' => $purchasedServices, 'forcefulDefault' => $forcefulDefault);

		//load the template and send the response
		$response 		= $this->_loadTemplate('available_addons', $args);
		$ajaxResponse 	= PayplansFactory::getAjaxResponse();
		$ajaxResponse->addAssign($appendTo,'innerHTML', $response );
		$ajaxResponse->sendResponse();
	}

	/**
	 * when user done with purchasing then collect them and charge them to invoices
	 * @return ajaxresponse page is refresh to show the modified changes on invoice
	 * @since 3.2	
	 */
	public function onPayplansPlanaddonsCalculate()
	{
		$eventArgs 	= JFactory::getApplication()->input->get('event_args', array(), 'array');	
		
		$invoiceId 	= PayplansHelperUtils::getIdFromKey($eventArgs['invoice_key']);
		$invoice 	= PayplansInvoice::getInstance($invoiceId);
		
		$planAddons = $eventArgs['plan_addons'];

		$forcefulDefault 	= $this->params->get('forceful_default', 0);

		if ($forcefulDefault) {
			// Chcek for available default addons
			$defaultAddons = $this->getDefaultAddons($invoice);

			if (!empty($defaultAddons) ) {
				foreach ($defaultAddons as $add ) {
					if(!in_array($add, $planAddons))	{
						$planAddons[] = $add;
					}
				}
			}
		}

		$this->calculateCharges($invoice, $planAddons);
		
		//refresh the page to see effect of these applied modifiers
        $ajaxResponse = PayplansFactory::getAjaxResponse();
        $ajaxResponse->addScriptCall('window.location.reload(true);');
        $ajaxResponse->sendResponse();

	}

	/**
	 * first make entries in planaddons stats then apply then on invoices
	 * @since 3.2
	 */
	protected function calculateCharges(PayplansInvoice $invoice, $addOnServices = array())
	{
        //insert all the purchase to planaddons stats
        $this->doEntryServicesStats($invoice, $addOnServices);
		//calculate all modifiers
        $this->createInvoiceModifiers($invoice);

		return true;
	}

    /**
     * do entry in service stats according to purchases services.
     * firstly delete all entries related to this particular invoice then create new entries, it solves the modification in add services.
     * in this stats we replicate the service data casue admin may change it later on after puchase, so current stats may also get change,
     * so to avoide this thing we just replicate this data.
     * @since 3.2 
     * @param PayplansInvoice $invoice
     * @param array $addOnServices
     */
    protected function doEntryServicesStats(PayplansInvoice $invoice, $addOnServices = array())
    {
        $invoiceId 		= $invoice->getId();
        $userId    		= $invoice->getBuyer();
        $isRecurring 	= $invoice->isRecurring();

        //delete all entries that are related to this invoice
        $model = PayplansFactory::getInstance('planaddonsstats', 'model');
        $model->deleteMany(array('reference' => $invoiceId ));

        //create new entries for each purchased
        foreach ($addOnServices as $service) {

            $service = PayplansPlanaddons::getInstance($service);
            //create new entry according to new details
            $newEntry = PayplansPlanaddonsstats::getInstance();
            $newEntry->set('user_id', $userId);
            $newEntry->set('planaddons_id', $service->get('planaddons_id', 0));
            $newEntry->set('title', $service->get('title', ''));
            $newEntry->set('price', $service->get('price', ''));
            $newEntry->set('price_type', $service->get('price_type', 0));
            $newEntry->set('addons_condition', $service->get('addons_condition', PAYPLANS_PLANADDONS_ONETIME));
            $newEntry->set('reference', $invoiceId);
            $newEntry->set('status', 0);
            $newEntry->set('purchase_date', new XiDate('now'));
            $newEntry->set('params', $service->get('params', null));
            $newEntry->save();
        }

        return true;
    }

    /**
     * 
    * createInvoiceModifiers collect all planaddons an apply them on the invoice. 
    * apply them individual not collectly becasue when some of service may be in percentage or some may be fixed
    * or some service may be recurring or some may not.
    * @since 3.2
    * @return boolean
     */
    protected function createInvoiceModifiers(PayplansInvoice $invoice)
    {
        $invoiceId 	= $invoice->getId();
        $userId 	= $invoice->getBuyer();
        $orderId 	= $invoice->getObjectId();

        //delete all previously attached addon modifiers
        $modal 	= PayplansFactory::getInstance('modifier', 'model');
        $type 	= PAYPLANS_PLANADDONS_MODIFIER;
        $modal->deleteMany(array('invoice_id' => $invoiceId, 'type' => "'$type'" ));

        //get all purhased services
        $modal 				= PayplansFactory::getInstance('planaddonsstats', 'model');
        $purchasedServices 	= $modal->loadRecords(array('reference' => $invoiceId));

        //add each service as modifier on this invoice
        foreach ($purchasedServices as $service) {

			$statsParams 	= new XiParameter($service->params);
			//if nothing is available then neither discountale nor taxable
			$applicability 	= $statsParams->get('applicability','PERCENT_NON_TAXABLE');
			$applicability 	= constant("PayplansModifier::$applicability");
			
			//Note : if we do check service condition in doEntryServiceStats instead of createInvoiceModifiers,
			//we can skip to create servicestats for which services is not applicable, if we can skip to apply.
			//think of these conditions after adding any new service condition
			
			$condition  = ( $service->addons_condition == PAYPLANS_PLANADDONS_ONETIME )
									? PayplansModifier::FREQUENCY_ONE_TIME
									: PayplansModifier::FREQUENCY_EACH_TIME;
				
						
            $data = array(
                'user_id' => $userId,
                'order_id' => $orderId,
                'invoice_id' => $invoiceId,
                'amount' => $service->price,
                'type' => PAYPLANS_PLANADDONS_MODIFIER,
                'reference' => $service->planaddons_stats_id,
                'message' => $service->title,
                'percentage' => $service->price_type,
                'frequency' => $condition,
                'serial' => $applicability,
            );

            PayplansHelperModifier::create($data);
           
			// If addon is one time applicable , then recurring plan convert to recurring + 1 trial 
         	if($service->addons_condition == PAYPLANS_PLANADDONS_ONETIME && $invoice->isRecurring() == PAYPLANS_RECURRING)
			{		
				$invParams 	= $invoice->getParams()->toArray();
				$invoice->setParam('expirationtype', 	'recurring_trial_1');
				$invoice->setParam('recurrence_count', 	($invParams['recurrence_count']> 0 ) ? $invParams['recurrence_count']-1 : 0);
				$invoice->setParam('trial_price_1', 	$invParams['price']);
				$invoice->setParam('trial_time_1', 		$invParams['expiration']);
				
				$invoice->refresh();
				$invoice->save();
			}
		
        }

        return true;
    }
    
    /**
     * here increased the cosumed counter to one, only after successfull payment
     * we can also work here for renewal and upgrade thing
    * onPayplansInvoiceAfterSave 
    * @since 3.2
    * @return boolean
     */
    public function onPayplansInvoiceAfterSave($prev, PayplansInvoice $new)
	{
		if (empty($prev) && $new->getStatus() == PayplansStatus::NONE) {

			$isRenewed = $new->getParam('isRenewed', 0);
			
			if (1 == $isRenewed) {
				return true;
			}

			// Chcek for available default addons
			$defaultAddons = $this->getDefaultAddons($new);
			
			if (empty($defaultAddons) ) {
				return true;
			}
			
			$this->calculateCharges($new, $defaultAddons);
			
			return true;
		}
		
		// only change the consumed quantity only after invoice paid
		if ($new->getStatus() != PayplansStatus::INVOICE_PAID) {
			return true;
		}

		$modal 				= PayplansFactory::getInstance('planaddonsstats', 'model');
		$purchasedServices 	= $modal->loadRecords( array( 'reference' => $new->getId() ) );

		foreach ($purchasedServices as $purchasedService) {

			$service 	= PayplansPlanaddons::getInstance($purchasedService->planaddons_id);
			$consumed 	= $service->get('consumed', 0);
			$service->set('consumed', ($consumed + 1) );
			$service->save();
		}

		return true;
	}

    /**
	 * for showing plugin configuration on apps screen 
     * @since 3.2
     */
    public function onPayplansGetPlugin()
    {
    	$xml = dirname(__FILE__).DS.'planaddons.xml';
    	if (file_exists($xml)) {
    		$xmlContent = simplexml_load_file($xml);
    	}
    	else {
    		$xmlContent = null;
    	}
    	return array('name'   => 'planaddons',
    					'value' =>$xmlContent);
    }
    
	//this function will provide the id's of default addons.
    public function getDefaultAddons(PayplansInvoice $invoice)
    {
    	$defaultAddons = array();
    	
    	//apply default addon only on first invoice otherwise it will get attached to recurring also
    	$isRenewed = $invoice->getParam('isRenewed', 0);

    	if ($invoice->getCounter() != 1 && 1 != $isRenewed) {
    		return $defaultAddons;
    	}
    	
    	$invoicePlans 		= $invoice->getPlans();
    	
    	$model 				= PayplansFactory::getInstance('planaddons', 'model');
    	$availableAddons 	= $model->loadAvailableServices($invoicePlans );
    	
    	if (empty($availableAddons)) {
    		return $defaultAddons;
    	}

    		
    	foreach ($availableAddons as $addon) {
    	
    		$params 	= new XiParameter($addon->params);
    		$default  	= $params->get('default', 0);
    	
    		if ($default) {
    			$defaultAddons[] = $addon->planaddons_id;
    		}
    	}

    	return $defaultAddons;
    }
    
	//need to handle the case of renewal cause, there is no way other than this to detect the renewed invoice
    public function onPayplansSubscriptionAfterRenewalInvoiceCreation($subscription, $newSubscription, $order, $invoice)
    {
    	$invoice->setParam('isRenewed', 1);
    	
    	$invoice->save();
    	
    	// Chcek for available default addons
    	$defaultAddons = $this->getDefaultAddons($invoice);
    		
    	if (empty($defaultAddons) ) {
    		return true;
    	}
    		
    	$this->calculateCharges($invoice, $defaultAddons);
    	
    	return true;
    }

	//Remove the associated addons while deleting user. 
	public function onUserBeforeDelete($user)
	{
		$userId 	 = $user['id'];
		$planAddons  = PayplansFactory::getInstance('planaddonsstats', 'model');
		$planAddons->deleteMany(array('user_id' => $userId));
	}

}
